#include <v8.h>
#include "tidy.h"
#include "buffio.h"

using namespace v8;

const int ENOMEM = -12;

char* newString(unsigned int length) {
  char *result = (char *)malloc(length*sizeof(char));
  int i;
  for(i=0;i<length;i++) {
    result[i] = 0;
  }
  return result;
}

Handle<Value> tidyString(const Arguments& args) {
  int rc;
  int len = args.Length();
  if(len < 1) return Undefined();
  String::Utf8Value content(args[0]); 
  char *html = *content;
  unsigned int resultLength = 32;
  char *result = newString(resultLength);
  TidyDoc doc = tidyCreate();
  TidyBuffer errorBuffer;
  tidyBufInit(&errorBuffer);

  tidyOptSetBool(doc, TidyXhtmlOut, yes);
  tidyOptSetBool(doc, TidyQuiet, yes);
  tidyOptSetBool(doc, TidyHideComments, yes);
  tidyOptSetBool(doc, TidyForceOutput, yes);
  tidySetErrorBuffer(doc, &errorBuffer);

  tidyParseString(doc, html);
  tidyCleanAndRepair(doc);
  tidyRunDiagnostics(doc);
  fprintf(stdout, "converting %d\n", strlen(html));
  //ENOMEM signifies we need to allocate more space
  //the amount of required space is stored in resultLength
  if((rc = tidySaveString(doc, result, &resultLength)) == ENOMEM) {
    fprintf(stdout, "Giving more space\n");
    free(result);
    result = newString(resultLength+1);
    rc = tidySaveString(doc, result, &resultLength);
  }
  if(rc >= 0) {
    fprintf(stdout, "Diagnostics:\n%s", errorBuffer.bp);
    fprintf(stdout, "Results:'%d'\n", strlen(result));
  } else {
    fprintf(stderr, "An error has occurred tidying input");
  }
  tidyRelease(doc);
  tidyBufFree(&errorBuffer);
  return String::New(result);
}

extern "C" void init(Handle<Object> target) {
  HandleScope scope;
  target->Set(String::New("tidyString"), FunctionTemplate::New(tidyString)->GetFunction());
}
